/* Copyright 2017 Beetroot design group, beetroot.gr

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

function myScript(thisObj) {
	function myScript_buildUI(thisObj) {
		var myPanel = (thisObj instanceof Panel) ? thisObj : new Window("palette", "Test", undefined, {
			resizeable: true
		});

		res = "panel{alignment:'fill',justify:'center',\
						  btn1: Button{text:'Flowtype Instance'},\
						  btn2: Button{text:'Find Loop Point'}\
				 }";

		myPanel.grp = myPanel.add(res);

		//Defaults
		myPanel.grp.btn1.onClick = function () {
			main();
		}
		myPanel.grp.btn2.onClick = function () {
			findLoop();
		}

		myPanel.layout.layout(true);
		return myPanel;
	}

	var myScriptPal = myScript_buildUI(thisObj);

	if (myScriptPal != null && myScriptPal instanceof Window) {
		myScriptPal.center();
		myScriptPal.show();
	}

}
myScript(this);

function findLoop() {
	var myComp = app.project.activeItem;
	var s = myComp.selectedLayers[0];
	var effectsGroup = s("Effects");
	var selectedEffects = new Array();
	if (effectsGroup !== null) {
		for (var i = 1; i <= effectsGroup.numProperties; i++)
			if (effectsGroup.property(i).selected)
				selectedEffects[selectedEffects.length] = effectsGroup.property(i);
	}
	if (selectedEffects[0] !== null) {
		var speed = selectedEffects[0].property("Speed").value;
		var isCurved = selectedEffects[0].property("Use Curved Motion").value;
		if (isCurved == 1) {
			var fps = myComp.frameRate;
			var t = Math.round(4.0 * fps / speed) - 1;
			myComp.time = currentFormatToTime(t, fps);
		} else {
			var fps = myComp.frameRate;
			var t = Math.round(4.0 * fps / speed) - 1;
			myComp.time = currentFormatToTime(t, fps);
		}
	}
}

function main() {
	var thisScript = new File($.fileName);
	var containingFolder = new Folder(thisScript.parent.absoluteURI);
	var myComp = app.project.activeItem;
	var s = myComp.selectedLayers[0];
	var effectsGroup = s("Effects");
	var selectedEffects = new Array();
	if (effectsGroup !== null) {
		for (var i = 1; i <= effectsGroup.numProperties; i++)
			if (effectsGroup.property(i).selected)
				effectsGroup.property(i).selected = false;
	}
	var thePreset = File(containingFolder.absoluteURI + "/Flowtype.ffx");
	if (s.property("sourceText") !== null) { // Or "Source Text", "text", or "Text"

		app.beginUndoGroup("Flowtype");
		s.applyPreset(thePreset);
		fxName = getLastEffectName(s);
		s.property("Effects")(fxName).selected = false;

		var transformAmountExpression = 'if (textTotal>2) {\
						if (effect("' + fxName + '")("Auto") == 0 ) {\
							t = effect("' + fxName + '")("Time");\
						} else {\
							t = time*effect("' + fxName + '")("Speed");\
						}\
					\
						f = effect("' + fxName + '")("Master Frequency");\
					\
						if (effect("' + fxName + '")("Use Curved Motion") == 0) {\
							finalTime = t+(effect("' + fxName + '")("Time Shift for auto"));\
							step = (textIndex-1)/(textTotal-1)*f;\
							theta = step+finalTime;\
							finalValue = Math.abs(1-Math.abs(((theta)%4)-2))*selectorValue;\
						} else {\
							//Curve 4*framerate/speed\
							ts = effect("' + fxName + '")("Time Shift for auto");\
							finalTime = (t+ts)*Math.PI;\
							step = (textIndex-1)/(textTotal-1)*f*Math.PI;\
							theta = step+finalTime;\
							finalValue = (Math.cos(step+finalTime)+1)/2*selectorValue;\
						}\
					} else {\
							finalValue = 0;\
					}\
					finalValue';
		var angleAmountExpression = 'if (textTotal>2) {\
						if (effect("' + fxName + '")("Auto") == 0 ) {\
							t = effect("' + fxName + '")("Time");\
						} else {\
							t = time*effect("' + fxName + '")("Speed");\
						}\
					\
						f = effect("' + fxName + '")("Master Frequency");\
					\
						if (effect("' + fxName + '")("Use Curved Motion") == 0) {\
							finalTime = t+(effect("' + fxName + '")("Time Shift for auto"));\
							step = (textIndex-1)/(textTotal-1)*f;\
							theta = step+finalTime;\
							finalValue = (1-Math.abs(((theta)%4)-2))*selectorValue;\
						} else {\
							ts = effect("' + fxName + '")("Time Shift for auto");\
							finalTime = (t+ts)*Math.PI/2;\
							step = (textIndex-1)/(textTotal-1)*f*Math.PI/2;\
							theta = step+finalTime;\
							finalValue = (-1*Math.cos(step+finalTime))*selectorValue;\
						}\
					} else {\
							finalValue = 0;\
					}\
					finalValue';

		//TRANSFORMS
		var transformAnim = s.Text.Animators.addProperty("ADBE Text Animator");
		transformAnim.name = "Transforms controllers";
		var transformSelector = transformAnim.property("ADBE Text Selectors").addProperty("ADBE Text Expressible Selector");
		transformSelector.amount.expression = transformAmountExpression;
		var transformHelper;
		transformHelper = transformAnim.property("ADBE Text Animator Properties").addProperty("ADBE Text Scale 3D");
		transformHelper.expression = 'h= effect("' + fxName + '")("Scale H");\ v= effect("' + fxName + '")("Scale V");\ [h,v];';
		transformHelper = transformAnim.property("ADBE Text Animator Properties").addProperty("ADBE Text Position 3D");
		transformHelper.expression = 'h= effect("' + fxName + '")("Position X");\ v= effect("' + fxName + '")("Position Y");\ [h,v,0];';
		transformHelper = transformAnim.property("ADBE Text Animator Properties").addProperty("ADBE Text Tracking Amount");
		transformHelper.expression = 'effect("' + fxName + '")("Tracking")';

		//ANGLULAR
		var angleAnim = s.Text.Animators.addProperty("ADBE Text Animator");
		angleAnim.name = "Angular controllers";
		var angleSelector = angleAnim.property("ADBE Text Selectors").addProperty("ADBE Text Expressible Selector");
		angleSelector.amount.expression = angleAmountExpression;
		var angleHelper;
		angleHelper = angleAnim.property("ADBE Text Animator Properties").addProperty("ADBE Text Skew");
		angleHelper.expression = 'effect("' + fxName + '")("Skew")*-1.0';
		angleHelper = angleAnim.property("ADBE Text Animator Properties").addProperty("ADBE Text Rotation");
		angleHelper.expression = 'effect("' + fxName + '")("Rotation")*-1.0';
		s.selected = true;

		app.endUndoGroup();
	} else {
		alert("Layer has no Text")
	}
}

function getLastEffectName(layerVar) {
	var numEffects = layerVar.property("Effects").numProperties;
	return layerVar.property("Effects")(numEffects).name;
}
